require "timelocal.pl";
###############################################################################
# wanstat.pl v1.47 10/04/2000 .
# Log analyzer for program wanstat.nlm
# Author : Michael V. Petrushin (petm@rarus.ru). Moscow. Russia
# You can find WanStat.nlm on http://www.officenet.hu/
###############################################################################
#
# Before running this program, you must run wanstat.nlm, that will create log 
# files to this program.
# Add "load wanstat.nlm N TXT" into autoexec.ncf your server, and after sometime
# with this program you can see statistic on interface N in view, more comfortable 
# than txt logs.
#
# To use this program as SCRIPT on Netware, put this file in perlroot directory.
# By default in Netware Enterprise Server perlroot="/novonyx/suitespot/docs/lcgi/perl/"
# Change $servername and $wanlink parameters and run it from client browser,
# for example: http://you_server/perl/wanstat.pl?last=1
# By default script show "by hour", "today" statistic.
#
# If you use it as console PROGRAM on Netware, put this file in /perl/scripts
# directory and change $servername, $wanlink and $outfile parameters.
# After than you can run program, for example from console or cron.nlm using 
# next syntax: "perl wanstat.pl all=1 --noscreen"
#
# To run as console program you MUST enter command line parameters.
# While executing html-file will be created (see $outfile parameter) and you
# can view this file in any browser.
# Available command line parameters (also available as script parameters after ?):
# from=yyyymmdd - from data (ex. 20000302 is 1 march 2000);
# to=yyyymmdd   - to data (format similar);
#                 if you enter "FROM" and doesn't enter "TO",
#                 statistic will be generated from "FROM" to today.
#                 if you enter "TO" and doesn't enter "FROM",
#                 statistic will be generated on day, defined in "TO" parameter
# last=n        - analyzing last n days/months.
#                 (by default last=1, days or months - consist from "type" value);
# all=1         - analyzing all logs (by default all=0);
# type=DAY      - out statistics "by day" or "by hour" (by default type=HOUR).
# Parameters must be separated by using '&' symbols, ex.: "wanstat.pl type=DAY&all=1"
# Some parameters not available both (ex. if FROM greate than TO).
#
# Tested under Active Perl 5.21 and Netware 5 Perl NDK 333 (January 2000).
# You can download last version Perl NDK for Netware from http://developer.novell.com/
# If you see message "Can`t locate timelocal.pl in @INC at wanstat.pl line 1" 
# after run program with "--noscreen" parameter you MUST update you version perl.
###############################################################################

$servername="CORDON";    # You server name. This field can be empty;
$wanlink=128;            # You wan bandwidth in KBits per second; required;
$outfile="out.htm";      # Output html file name, REQUIRED if program 
                         # used as console program. You can enter full path.
                         # for example: /novonyx/suitespot/docs/out.htm;
$logdir="/etc/wanstat";  # Path to directory where logs wanstat.nlm located;
$rxcolor="blue";         # Receive - color blue;
$txcolor="red";          # Transmit - color red.


###############################################################################
#                                                                       
# You should NOT need to change anything beyond this point, but you     
# can read through the script if you're interested in what it's doing.  
#                                                                       
###############################################################################

$ver = "1.47";
$type="HOUR";            # TYPE value by default
@month_name=qw(January February March April May June July August September October November December);

$last=1;
$max=int($wanlink/8*3600);
$programname=$0;
($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime(time);
$mon++;
$year+=1900;
if ($mon < 10) {$mon="0".$mon};
if ($mday < 10) {$mday="0".$mday};
$todata = $data = $today = "$year"."$mon"."$mday";
$ll=substr($logdir,-1);
if (($ll eq '/') or ($ll eq '\\')) {
  substr($logdir,-1)="";
}

# input parameters ... by GET method (run as script) or as command line parameters (run as program).
$query_string = $ENV{'QUERY_STRING'}; 
$runtype="script"; 
# if arguments in command line exists - run as program (else as script).
if ((@ARGV) ) {
  $query_string=join("",@ARGV);
  $runtype="noscript";
}
@indata=split(/&/,$query_string);
foreach (@indata) {
  ($key,$val)=split(/=/,$_);
  $val =~ tr/+/ /;
  $val =~ s/%([\dA-Fa-f][\dA-Fa-f])/pack ("C", hex ($1))/eg;
  $key =~ tr/a-z/A-Z/;
  $data{$key}=$val;
}

PrintHeader();

if ($data{LAST} ne "") {$last=int($data{LAST})};
if ($data{TO} ne "") {$todata=$data{TO}};
$fromdata=DecData(int($todata),$last-1);
if ($data{FROM} ne "") {$fromdata=$data{FROM}};
if ($data{TYPE} eq "DAY") {$type="DAY"};
if ($todata > $today) {$todata=$today};

# read to array @files list all found logs from input directory
# and sort it by date
$mask=".+\.txt";          # mask to find wanstat.nlm logs (perl regular expression, not DOS mask :) )
opendir(LOGDIR,"$logdir/");
@files=sort grep { /$mask/i } readdir(LOGDIR);
closedir(LOGDIR);

init();
$txmonth=0;
$rxmonth=0;
$mcompleate=0;
$m=0;
$mfirst=1;

foreach $file (sort @files) {
  $f=substr($file,0,8);
  $y=substr($f,0,4);
  $mprev=$m;
  $m=substr($f,4,2);
  $d=substr($f,6,2);
  if ((($f >= $fromdata) and ($f <= DecData($todata,-1))) or ($data{ALL} == 1) or (($type eq "DAY") and (($m == $mon) or (abs($m-$mon) < $last))) ) {
    if (($mfirst == 1) and ($type ne "HOUR")) {
      print "<center><b><font face=Arial size=4>$month_name[$m-1] $y</font></b></center>";
      $mfirst=0;
    };
    open (F, "$logdir/$file");
    while (<F>) {
      chomp;
      if ($_ ne "") {
        $prizn=1;
        $intprev=$int;
        $txprev=$tx;
        $rxprev=$rx;
        $dataprev=$data;
        $timeprev=$time;
        ($int, $t1, $tx, $t, $r1, $rx, $t, $at, $data, $time, $state) = split(/ /,$_);
        if (($intprev eq $int) and ($t1 eq "Tx") and ($r1 eq "Rx") and ($data=~/^\d{2}\/\d{2}\/\d{4}/) and ($time=~/^\d{2}:\d{2}:\d{2}/)) {
          if (($tx < $txprev) or ($rx < $rxprev)) {
            $txprev=$rxprev=0;
          }
          $dtx=$tx-$txprev;
          $drx=$rx-$rxprev;
          ($hour,$min,$sec) = split(/:/,$time);
          ($hourprev,$minprev,$secprev) = split(/:/,$timeprev);
          $dt=($hour-$hourprev)*60+$min-$minprev+($sec-$secprev)/60;
          if ($dt < 0) {
            $dt+=1440;
            $prizn++;
          };
          if ($dt == 0) {
#            $dtxh=$drxh=0
             next;
          } else {
            if ($hour != $hourprev) {
              $dtxh+=int(($dtx/$dt)*(60-$minprev-$secprev/60)+0.5);
              $drxh+=int(($drx/$dt)*(60-$minprev-$secprev/60)+0.5);
              $txh{$hourprev}=$dtxh;
              $rxh{$hourprev}=$drxh;
              $dtxh=int(($dtx/$dt)*($min+$sec/60)+0.5);
              $drxh=int(($drx/$dt)*($min+$sec/60)+0.5);
            } else {
              $dtxh+=$dtx;
              $drxh+=$drx;
            }
          }
          $txtotal=$txtotal+$dtx;
          $rxtotal=$rxtotal+$drx;
#          print " h=$hour hp=$hourprev dt=$dt dtxh=$dtxh drxh=$drxh dtx=$dtx drx=$drx $txprev\n";
        } elsif (($t1 ne "Tx") or ($r1 ne "Rx") or (!($data=~/^\d{2}\/\d{2}\/\d{4}/)) or (!($time=~/^\d{2}:\d{2}:\d{2}/))) {
          $data=$dataprev;
          $time=$timeprev;
          $tx=$txprev;
          $rx=$rxprev;
        }
        if ($prizn > 1) {PrintGraph()};
      }
    }
    close (F);
  }
}
if (($prizn == 1) and ($todata == $f)) {PrintGraph()};
if (($mcompleate != 0) and ($type ne "HOUR")) { PrintPeriodInfo()};
PrintFooter();


sub PrintGraph {
  if ($prizn > 1) {
    ($d,$m,$y) = split ('/',$dataprev);
  } else {
    ($d,$m,$y) = split ('/',$data);
  }
  if ($type eq "HOUR") {
    PrintData()
  };
  my($h,$hh,$t,$r,$dltx,$dlrx,$ll,$c1,$c2,$w1,$w2,$ws);
  foreach (sort keys(%txh)) {
    $h=$_;
    $hh=$h+1;
    if ($hh < 10) {$hh="0"."$hh"};
    $t=$txh{$_};
    $r=$rxh{$_};
    $dltx=int(($t/$max)*400+1.5);
    $dlrx=int(($r/$max)*400+1.5);
    if ($dltx < 4) {$dltx=4};
    if ($dlrx < 4) {$dlrx=4};
    $ww=400-$dltx-$dlrx;
    if ($type eq "HOUR") {
      $percent=int(($t+$r)/$max*100);
      print "<table cellspacing=1 cellpadding=0 border=0><tr><td>$h:00-$hh:00 </td><td width=$dlrx bgcolor=$rxcolor>&nbsp;</td><td width=$ww bgcolor=cyan>&nbsp;</td><td width=$dltx bgcolor=$txcolor>&nbsp;</td>";
      print "<td>&nbsp; <font color=$rxcolor>Receive</font>=$r Kb,<font color=$txcolor>Transmit</font>=$t Kb ($percent%)</td></tr></table>";
    }
  }
  $txtotal=int($txtotal+0.5);
  $rxtotal=int($rxtotal+0.5);
  $percent=int(($txtotal+$rxtotal)/$max/24*100);

  $dd="$y"."$m"."$d";
  if ($type eq "HOUR") {
    print "<b><font face=Tahoma>Total: <font color=$rxcolor>Receive</font>=$rxtotal Kb, <font color=$txcolor>Transmit</font>=$txtotal Kb, Daily utilization=$percent%</font></b>\n\n"
  } else {
    if ($m ne $mprev) {
      PrintPeriodInfo();
      print "\n<center><b><font face=Arial size=4>$month_name[$m-1] $y</font></b></center>";
      $txmonth=0;
      $rxmonth=0;
      $mcompleate=0;
    }
    $daytxtraffic{$dd}=$txtotal;
    $dayrxtraffic{$dd}=$rxtotal;
    $txtot=int(($txtotal/$max/24)*400+1.5);
    $rxtot=int(($rxtotal/$max/24)*400+1.5);
  if ($txtot < 4) {$txtot=4};
  if ($rxtot < 4) {$rxtot=4};
    $ww=400-$txtot-$rxtot;
    $daytraffic{$dd}="<table cellspacing=1 cellpadding=0 border=0><tr><td>";
    if ($runtype eq "script") {
      $daytraffic{$dd}="$daytraffic{$dd}"."<a href=$programname?TO=$y$m$d>$d-$m-$y</a> ";
    } else {
      $daytraffic{$dd}="$daytraffic{$dd}"."$d-$m-$y ";
    }
    $daytraffic{$dd}="$daytraffic{$dd}"."</td><td width=$rxtot bgcolor=$rxcolor>&nbsp;</td><td width=$ww bgcolor=cyan>&nbsp;</td><td width=$txtot bgcolor=$txcolor>&nbsp;</td><td>&nbsp; <font color=$rxcolor>Receive</font>=$rxtotal <font color=$txcolor>Transmit</font>=$txtotal ($percent%)</td></tr></table>";
    print "$daytraffic{$dd}";
    $txmonth+=$txtotal;
    $rxmonth+=$rxtotal;
  }
  init();
  $mcompleate++;
}

sub PrintPeriodInfo {
  my ($t,$r,$persent);
  $percent=int(($txmonth+$rxmonth)/$max/24/$mcompleate*100);
  $t=int($txmonth/1024+0.5);
  $r=int($rxmonth/1024+0.5);
  print "<b><font face=Tahoma>Total: <font color=$rxcolor>Receive</font>=$r Mb, <font color=$txcolor>Transmit</font>=$t Mb, Average utilization=$percent%</font></b>\n";
}

sub PrintData {
  my ($dd,$dd1,$dd11,$dd2,$dd22,$f);
  if ($type eq "HOUR") {
    $dd=$dd2=$dd1="$y"."$m"."$d";
    $dd1=DecData($dd1,DifData($fromdata,$todata)+1);
    $dd11=DecData($dd,1);
    $dd2=DecData($dd2,-1);
    $dd22=DecData($dd2,DifData($todata,$fromdata));
    $f=substr(@files[0],0,8);
    print "<center>";
    if (($dd1 >= $f) and ($fromdata eq $dd) and ($runtype eq "script")) {print "<font face=Tahoma><a href=$programname?FROM=$dd1&TO=$dd11>previous</a></font> "};
    print "<b><font face=Arial size=4>$d-$month_name[$m-1]-$y</font></b>";
    if (($dd2 <= $today) and ($dd eq $todata) and ($runtype eq "script")) {print " <font face=Tahoma><a href=$programname?FROM=$dd2&TO=$dd22>next</a></font>"};
    print "</center>"
  }
}


sub max {
 my ($m) = 0;
 foreach (@_) {
   if ($_ > $m) {$m=$_};
 }
 return $m;
}

sub init {
  $prizn=0;
  $dt=0;       
  $dtx=0;      
  $drx=0;      
  $dtxh=0;
  $drxh=0;
  $txtotal=0;  
  $rxtotal=0;  
  %txh=();     
  %rxh=();     
}

sub PrintHeader {
  if ($runtype ne "script") {
    open (OUT, ">$outfile");
    select(OUT);
  } else {
    print "Content-type: text/html\n\n";
  }
  print "<html>\n<head>\n";
  print "<title>Receive/Transmit statistics.";
  if ($servername ne "") {print " Server: $servername"};
  print "</title>\n<style type=\"text/css\"><!--td {font-size:12px; font-family:Tahoma}--></style></head>\n<body>\n";
  if ($runtype eq "script") {
    PrintLinks();
  }
  print "<pre>\n";
}

sub PrintFooter {
  print "</pre>\n";
  if ($runtype eq "script") {
    PrintLinks();
  }
  print "<center><font face=Tahoma color=dimgray size=-2><br>Script <font color=black>$programname</font> v$ver writen by <a href=\"mailto:petm\@rarus.ru?Subject=About you script wanstat.pl v$ver\">Michael V. Petrushin</a></font></center>\n";
  print "</body></html>\n";
  if ($runtype ne "script") {
    select(STDOUT);
    close(OUT);
  }
}


sub PrintLinks {
  print "<center><a href=$programname?last=1>today</a> | ";
  print "<a href=$programname?last=2>last 2 days</a> | ";
  print "<a href=$programname?last=7>last week</a> | ";
  print "<a href=$programname?all=1>all</a> | ";
  print "<a href=$programname?type=DAY>last month by day</a> | ";
  print "<a href=$programname?type=DAY&last=2>last two months by day</a> | ";
  print "<a href=$programname?type=DAY&all=1>all logs by day</a></center>\n";
}


sub DecData {
  my ($y,$m,$d,$orgdata,$diff,$newdata,$sec,$min,$hour,$yday,$isdst,$wday);
  my $data = shift;
  my $dd = shift;
  $y=substr($data,0,4);
  $m=substr($data,4,2);
  $d=substr($data,6,2);
  $orgdata=timelocal(0,0,0,$d,--$m,$y);
  $newdata=$orgdata - 86400 * $dd;
  ($sec,$min,$hour,$d,$m,$y,$wday,$yday,$isdst) = localtime($newdata);
  $m++;
  $y+=1900;
  if ($d < 10) {$d="0".$d};
  if ($m < 10) {$m="0".$m};
  $data="$y"."$m"."$d";
  return $data;
}

sub DifData {
  my ($y1,$m1,$d1,$y2,$m2,$d2,$orgdata1,$orgdata2,$diff);
  my $data1 = shift;
  my $data2 = shift;
  $y1=substr($data1,0,4);
  $m1=substr($data1,4,2);
  $d1=substr($data1,6,2);
  $y2=substr($data2,0,4);
  $m2=substr($data2,4,2);
  $d2=substr($data2,6,2);
  $orgdata1=timelocal(0,0,0,$d1,--$m1,$y1);
  $orgdata2=timelocal(0,0,0,$d2,--$m2,$y2);
  $diff=$orgdata2-$orgdata1;
  $data1=int($diff/86400);
  return $data1;
}